import sys
import subprocess
import urllib.error
from urllib import request
import json
import click

version = "1.1.4 (beta)"


def latest_version():
    res = request.urlopen("https://api.github.com/repos/islam-kamel/django-start/tags")
    version_name = json.load(res)[0]["name"]
    version_int = version_name.split(".")
    version_int[-1] = version_int[-1].split("-")[0]
    version_int = [int(num) for num in version_int]
    return version_name, version_int


def current_version():
    return [1, 1, 4]


def check_available():
    try:
        ver_name, ver_int = latest_version()
        current = current_version()
        if sum(ver_int) > sum(current):
            print(f"New Update Available {ver_name}")
            return True

    except urllib.error.URLError:
        sys.exit(1)
        pass


@click.command()
@click.option("--update")
@click.option('--check-update')
def main(update, check_update):
    """
    Display current versio,
    check new update and update django-start
    """
    if check_update:
        return check_available()

    if update:
        return subprocess.call(
            f"{sys.executable} -m pip install --upgrade django-start-automate",
            shell=True,
        )

    print(version)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import pathlib

import click
from djstart_interface import DjangoStart
from hellper import create_env


@click.command()
@click.argument("project_name", type=str)
@click.argument("app_name", type=str)
@click.option(
    "-n",
    "--name",
    default="env",
    help="Custom Environment Name",
    type=lambda p: pathlib.Path(p).absolute(),
)
@click.option("-v", "--virtualenv", help="Install Environment Is Deprecated")
def main(**kwargs):
    """
    Prepare a new Django project quickly and automatically and,
    more wonderful things that save you time By django start
    """
    if kwargs["virtualenv"]:
        click.secho(
            "Please Don't Use This Option is Deprecated By Default Created Virtual Environment",
            fg="white",
            bg="red",
        )
        click.secho("Creating a virtual environment is a best practice!", fg="green")

    create_env(kwargs["name"])
    django_start = DjangoStart(
        app_name=kwargs["app_name"], core_name=kwargs["project_name"]
    )
    django_start.setup_project()
    django_start.setup_app()

if __name__ == "__main__":
    main()



import os
import click
import subprocess
import platform
import sys
from string import Template


def warn_stdout(message):
    print(f"⚠️ \033[93mWARNING: {message}\033[0m")


def create_env(env_name_path):
    click.secho("\U0001F984 Create Environment...", fg="blue")
    subprocess.call(
        f"{sys.executable} -m venv {env_name_path}",
        stdout=subprocess.DEVNULL,
        stderr=subprocess.STDOUT,
        shell=True,
    )
    if platform.system() == "Windows":
        os.environ.setdefault(
            "PYTHONPATH", f"{env_name_path}{os.sep}Scripts{os.sep}python.exe"
        )
        os.environ.setdefault(
            "DJANGOADMIN", f"{env_name_path}{os.sep}Scripts{os.sep}django-admin.exe"
        )
    else:
        os.environ.setdefault(
            "PYTHONPATH", f"{env_name_path}{os.sep}bin{os.sep}python3"
        )
        os.environ.setdefault(
            "DJANGOADMIN", f"{env_name_path}{os.sep}bin{os.sep}django-admin"
        )


def executable_python_command(command):
    try:
        proc = subprocess.call(
            f"{os.environ.get('PYTHONPATH')} {command}",
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT,
            shell=True,
        )
        if proc:
            if platform.system() != 'Windows':
                click.secho("check your installed python3-env", fg="white", bg="read")
            sys.exit(1)
    except KeyError:
        click.secho("Be sure to set up PYTHONPATH", fg="white", bg="read")
        sys.exit(1)


def executable_django_command(command):
    try:
        proc = subprocess.call(
            f"{os.environ.get('DJANGOADMIN')} {command}",
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT,
            shell=True,
        )
        if proc:
            sys.exit(1)
    except KeyError:
        click.secho("Be sure to set up DJANGOADMIN", fg="white", bg="read")
        sys.exit(1)


def upgrade_pip():
    executable_python_command("-m pip install --upgrade pip")


def install_dep():
    executable_python_command("-m pip install django")


def requirements_extract():
    executable_python_command("-m pip freeze > requirements.txt")


def build_view_func():
    """
    :param
    app_name
    html_file
    :return: str
    """
    s = Template(
        f"""def home(request):
    return render(request, '$app_name{os.sep}$html_file')
    """
    )
    return s


def build_views_urls():
    s = Template(
        """from django.urls import path
from . import views

urlpatterns = [
    path('', views.$view_name)
]
    """
    )
    return s


def generate_html():
    """
    :return:
    Html Code
    """

    s = """<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="UTF-8">
        <title>Django Start</title>
        <style>
            *{text-align: center}
            div{display:flex; justify-content:center}
        </style>
    </head>
    <body>
        <h1 style="text-align: center">Hello, Django-Start</h1>
        <a style="text-align: center" class="github-button" href="https://github.com/islam-kamel/django-start" data-color-scheme="no-preference: light; light: light; dark: dark;" data-size="large" data-show-count="true" aria-label="Star islam-kamel/django-start on GitHub">Django-Start</a>
    </body>
    <script async defer src="https://buttons.github.io/buttons.js"></script>
</html>
    """
    return s



import os
from hellper import (
    warn_stdout,
    executable_python_command,
    build_views_urls,
    build_view_func,
    generate_html,
)

import click


class AppManager:
    def __init__(self, *args, **kwargs):
        self.__app_name = kwargs.get("app_name", None)
        self.__workdir = os.getcwd() + rf"{os.sep}{self.app_name}"
        self.__views = self.workdir + rf"{os.sep}views.py"
        self.__urls = self.workdir + rf"{os.sep}urls.py"
        self.__templates = self.workdir + f"{os.sep}templates"
        self.python = os.environ.get("PYTHONPATH", None)
        self.__line_list = []

    @property
    def app_name(self):
        return self.__app_name

    @property
    def workdir(self):
        return self.__workdir

    @property
    def views(self):
        return self.__views

    @property
    def urls(self):
        return self.__urls

    @property
    def line_list(self):
        return self.__line_list

    @line_list.setter
    def line_list(self, value):
        self.__line_list = value

    def index(self, value):
        return self.line_list.index(value)

    def read_file(self, file):
        with open(file) as f:
            self.line_list = f.readlines()
            f.close()
        return self.line_list

    def update_lines_list(self, flag, value):
        self.line_list.insert(self.index(flag), value)

    def replace_line(self, index, value):
        self.line_list[index] = value

    def create_app(self):
        if self.app_name not in os.listdir(
            f"{os.sep}".join(self.workdir.split(os.sep)[:-1])
        ):
            click.secho(f"\U00002728 Create '{self.app_name}' App...", fg="blue")
            executable_python_command(f"manage.py startapp {self.app_name}")
        else:
            warn_stdout(f'"{self.app_name}" already exist!')

    def update_view(self):
        self.read_file(self.views)

        if "# Create your views here.\n" in self.line_list:
            click.secho(f"\U0001F304 Create '{self.app_name}' View...", fg="blue")
            code_of_block = build_view_func().substitute(
                app_name=self.app_name, html_file="index.html"
            )
            self.replace_line(self.index("# Create your views here.\n"), code_of_block)

            with open(self.views, "w") as f:
                f.write("".join(self.line_list))
                f.close()
        else:
            warn_stdout('"home" View is already created!')

    def create_urls(self):
        with open(self.urls, "w") as f:
            click.secho(f"\U0001F517 Create {self.app_name} URLs...", fg="blue")
            block_of_code = build_views_urls().substitute(view_name="home")
            f.write(block_of_code)
            f.close()

    def create_templates(self):
        click.secho(f"\U0001F389 Generate '{self.app_name}' Index Page...", fg="blue")

        if not os.path.exists(self.__templates):
            os.mkdir(self.__templates)

        self.__templates += f"{os.sep}{self.app_name}"
        if not os.path.exists(self.__templates):
            os.mkdir(self.__templates)
        with open(f"{self.__templates}{os.sep}index.html", "w") as f:
            f.write(generate_html())
            f.close()



from project_manager import ProjectManager


class DjangoStart(ProjectManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def setup_project(self):
        self.upgrade_pip()
        self.install_dep()
        self.create_project()
        self.requirements_extract()

    def setup_app(self):
        self.app_manager.create_app()
        self.update_settings()
        self.update_urls()
        self.app_manager.update_view()
        self.app_manager.create_templates()
        self.app_manager.create_urls()



import os
from app_manager import AppManager
from hellper import (
    warn_stdout,
    executable_django_command,
    install_dep,
    requirements_extract,
    upgrade_pip,
)
import click


class ProjectManager:
    def __init__(self, *args, **kwargs):
        self.__app_manager = AppManager(*args, **kwargs)
        self.__workdir = os.getcwd()
        self.__app_name = kwargs.get("app_name", None)
        self.__core_name = kwargs.get("core_name", None)
        self.__urls = self.__workdir + rf"{os.sep}{self.core_name}{os.sep}urls.py"
        self.__settings = (
            self.__workdir + rf"{os.sep}{self.core_name}{os.sep}settings.py"
        )
        self.python = os.environ.get("PYTHONPATH", None)
        self.django_admin = os.environ.get("DJANGOADMIN", None)
        self.__other_dependencies = []
        self.__line_list = []

    @property
    def app_manager(self):
        return self.__app_manager

    @property
    def python(self):
        return self.__python

    @python.setter
    def python(self, python_path):
        self.__python = python_path

    @property
    def django_admin(self):
        return self.__django_admin

    @django_admin.setter
    def django_admin(self, django_admin_path):
        self.__django_admin = django_admin_path

    @property
    def core_name(self):
        return self.__core_name

    @property
    def workdir(self):
        return self.__workdir

    @property
    def app_name(self):
        return self.__app_name

    @property
    def urls_path(self):
        return self.__urls

    @property
    def settings_path(self):
        return self.__settings

    @property
    def line_list(self):
        return self.__line_list

    @line_list.setter
    def line_list(self, value):
        self.__line_list = value

    def index(self, value):
        return self.__line_list.index(value)

    def read_file(self, file):
        with open(file) as f:
            self.__line_list = f.readlines()
            f.close()
        return self.__line_list

    def update_lines_list(self, flag, value):
        self.__line_list.insert(self.index(flag), value)

    def replace_line(self, index, value):
        self.__line_list[index] = value

    def update_settings(self):
        self.read_file(self.settings_path)
        if f"\t'{self.app_name}',\n" not in self.__line_list:
            click.secho("\U0001F527 Update Project Settings...", fg="blue")
            self.update_lines_list("]\n", f"\t'{self.app_name}',\n")
            with open(self.settings_path, "w") as f:
                f.write("".join(self.__line_list))
                f.close()
        else:
            warn_stdout(f'"{self.app_name}" is installed!')

    def update_urls(self):
        self.read_file(self.urls_path)
        view_path = f"\tpath('', include('{self.app_name}.urls')),\n"
        if view_path not in self.__line_list:
            self.replace_line(
                self.index("from django.urls import path\n"),
                "from django.urls import path, include\n",
            )
            self.update_lines_list("]\n", view_path)
            with open(self.urls_path, "w") as f:
                f.write("".join(self.__line_list))
                f.close()
        else:
            warn_stdout("Urls Already Updated")

    @staticmethod
    def upgrade_pip():
        click.secho("\U0001F4E6 Upgrade Pip...", fg="blue")
        upgrade_pip()

    @staticmethod
    def install_dep():
        click.secho("\U000023F3 Install Dependencies...", fg="blue")
        install_dep()

    @staticmethod
    def requirements_extract():
        click.secho("\U0001F4C3 Generate Requirements.txt...", fg="blue")
        requirements_extract()

    def create_project(self):
        if self.core_name not in os.listdir(self.workdir):
            click.secho(f"\U00002728 Create '{self.core_name}' Project", fg="blue")
            executable_django_command(f"startproject {self.core_name} .")
        else:
            warn_stdout(f'"{self.core_name}" already exist!')
